/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2015 Bucknell University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Li Li (ll024@bucknell.edu)
 */

#include "ns3/object.h"
#include "ns3/log.h"
#include "ns3/simulator.h"
#include "ns3/event-driven-collector.h"


NS_LOG_COMPONENT_DEFINE ("EventDrivenCollector");

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (EventDrivenCollector);

TypeId
EventDrivenCollector::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::EventDrivenCollector")
    .SetParent<Collector> ()
    .AddConstructor<EventDrivenCollector> ()
    .AddTraceSource ("Output",
                     "The double value received by the trace sink",
                     MakeTraceSourceAccessor (&EventDrivenCollector::m_output),
                     "ns3::DataCollectionObject::CollectorTraceCallback")
  ;
  return tid;
}

EventDrivenCollector::EventDrivenCollector ()
  : Collector ()
{
  NS_LOG_FUNCTION (this);
}

EventDrivenCollector::~EventDrivenCollector ()
{
  NS_LOG_FUNCTION (this);
}

void
EventDrivenCollector::TraceSinkDouble (double oldValue, double newValue)
{
  NS_LOG_FUNCTION (this << oldValue << newValue);
  if (!IsEnabled ())
    {
      NS_LOG_DEBUG ("Collector not enabled");
      return;
    }
  m_doubleValue = newValue;

  // Report data to downstream objects
  ReportData ();
}

void
EventDrivenCollector::ReportData (void)
{
  NS_LOG_FUNCTION (this);
  if (!IsEnabled ())
    {
      return;
    }
  // get current simulation time
  double time = Simulator::Now ().GetSeconds ();

  // report the time and the newly received value to downstream
  // objects
  m_output (time, m_doubleValue);
}

} // namespace ns3
